#!/usr/local/bin/python 
from Tkinter import *                              # get widget classes
from Dialog  import Dialog

class Hello(Frame):                                # an extended frame
    def __init__(self, parent=None):               # attach to top-level?
        Frame.__init__(self, parent)               # do superclass init
        self.pack()
        self.createWidgets()                       # attach frames/widgets
        self.master.title("Buttons and Menus")     # set window-manager info
        self.master.iconname("tkpython")           # label when iconified

    def createWidgets(self):
        self.makeMenuBar()
        Label(self, {'text':  'Hello menu/toolbar world', 
                      Pack:   {'padx': 30, 'pady':30} })
        self.makeToolBar()

    def makeToolBar(self):
        toolbar = Frame(self, {'cursor': 'hand2',
                               'relief': 'sunken', 'bd': 2,
                                Pack:     {'side': 'bottom', 'fill': 'x'} })
        Button(toolbar, 
                 {'text':   'Quit', 
                  'command': self.quit, Pack:{'side': 'right'}})
        Button(toolbar, 
                 {'text':   'Hello',
                  'command': self.greeting, Pack:{'side':'left'}})

    def makeMenuBar(self):
        self.menubar = Frame(self, {'relief': 'raised', 'bd': 2})
        self.menubar.pack({'side': 'top', 'fill': 'x'})
        self.menubar.tk_menuBar(self.fileMenu(), self.editMenu())

    def fileMenu(self):
        mbutton = Menubutton(self.menubar,
                        {'text':     'File', 
                         'underline': 0,
                          Pack:       {'side': 'left', 'padx': '1m'}})
        menu = Menu(mbutton)
        menu.add_command({'label': 'New...',  'command': self.notdone})
        menu.add_command({'label': 'Open...', 'command': self.notdone})
        menu.add_command({'label': 'Quit',    'command': self.quit})
        mbutton['menu'] = menu
        return mbutton

    def editMenu(self):
        mbutton = Menubutton(self.menubar, 	
                        {'text':     'Edit', 
                         'underline': 0,
                          Pack:       {'side': 'left', 'padx': '1m'}})
        menu = Menu(mbutton)
        menu.add_command({'label': 'Cut',    'command': self.notdone})
        menu.add_command({'label': 'Paste',  'command': self.notdone})
        menu.add_separator({})

        submenu = Menu(mbutton) 
        submenu.add_command({'label': 'Spam', 'command': self.notdone})
        submenu.add_command({'label': 'Eggs', 'command': self.greeting})
        menu.add_cascade({'label': 'Stuff', 'menu': submenu})

        menu.add_command({'label': 'Delete', 'command': self.greeting})
        menu.entryconfig(1, {"state" : "disabled"})
        mbutton['menu'] = menu
        return mbutton

    def greeting(self):
        Dialog(self, {'title': 'greeting', 
                      'text':  'Howdy', 
                      'default':0, 'bitmap':'', 'strings':('hi',)})

    def notdone(self):
        Dialog(self, {'title': 'Not implemented', 
                      'text':  'Not yet available', 
                      'default': 0, 'bitmap': 'error', 'strings': ('OK',)})
        
    def quit(self):
        ans = Dialog(self, {'title':  'Verify quit',
                            'text':   'Are you sure you want to quit? ',
                            'bitmap': 'question',
                            'default': 1,
                            'strings': ('Yes', 'No')})
        if ans.num == 0: Frame.quit(self)

if __name__ == '__main__':  Hello().mainloop()    # if I'm run as a script
